/*
<This program is a text editor ebable of transcription on text file.>
<Ce programme est un éditeur de texte capable de chiffrement sur des fichier texte.>
Copyright (C) 2008 Keven Chaussé
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <QProgressDialog>
#include <QInputDialog>
#include <QFileDialog>

#include "Crypteur.h"

const QString SPECIAL_CHAR("éèëêÉÈËÊàáâäÀÁÂÄöôòóÖÔÒÓîïìíÏÎÌÍûüùúÙÛÜÚçÇñÑ");

Crypteur::Crypteur(QString texte, FenCryptographe *parent) : FenCryptographe(parent)
{
    m_texte = new QString(texte);
    resultatDecryption = new QString;
    m_parent = parent;
    initialRandom();
}

void Crypteur::setQString(QString texte)
{
    m_texte = new QString(texte);
}

void Crypteur::setParent(FenCryptographe *parent)
{
    m_parent = parent;
}

int Crypteur::compteLettreAZaz()
{
    int grandeur = 0;
    for(int i = 0; i < m_texte->size(); i++)
    {
        int caractere = m_texte->at(i).toLatin1();
        if(((caractere >= 65) && (caractere <= 90)) || ((caractere >= 97) && (caractere <= 122)))
        grandeur++;
    }
    return grandeur;
}

void Crypteur::initialRandom()
{
    QTime midnight(0, 0, 0);
    qsrand(midnight.secsTo(QTime::currentTime()));
}

int Crypteur::getRandomInt(int min, int max)
{
     return (int) (min + ((float) qrand() / RAND_MAX * (max - min + 1)));
}

bool Crypteur::AZazOnly(QString* texte)
{
    bool AzOnly = true;
    for(int i = 0; i < texte->size(); i++)
    {
        int caractere = texte->at(i).toLatin1();

        if(((caractere < 65) || (caractere > 90)) && ((caractere < 97) || (caractere > 122)))
        AzOnly = false;
    }

    if(!AzOnly)
    {
        QMessageBox message(m_parent);
        message.setIcon(QMessageBox::Warning);
        message.setWindowTitle(tr("Entrée de la clé incorrecte"));
        message.setText(tr("La clé entrée est invalide. Veuillez en choisir une autre."));
        message.exec();
    }
    return AzOnly;
}

QList<int> Crypteur::comporteASCII(QString* texte)
{
    QList<int> listeASCII;

    for(int i = 0; i < texte->size(); i++)
    {
        if(SPECIAL_CHAR.contains(texte->at(i)))
        {
            listeASCII.append(i);
        }
    }
    return listeASCII;
}

bool Crypteur::convertionAZaz()
{
    QList<int> listeASCII = comporteASCII(m_texte);

    if(!listeASCII.isEmpty())
    {
        QMessageBox message(m_parent);
        message.setIcon(QMessageBox::Warning);
        message.setWindowTitle(tr("Convertion ASCII"));
        message.setStandardButtons(QMessageBox::Ok | QMessageBox::Abort);
        message.setText(tr("Des caractères de format ASCII non-compatible ont été trouvés. Ils seront convertie.\n(ex: 'é' et 'è' vont devenir 'e')"));

        bool sure = false;
        int boutonAppuyer = message.exec();
         int i = 0;
        QMessageBox verification(m_parent);
        QProgressDialog progress(tr("Convertion des caractères ASCII en cours..."), tr("Annuler"), 0, listeASCII.size(), m_parent);
        progress.setWindowModality(Qt::WindowModal);
        while(!sure)
        {
            switch(boutonAppuyer)
            {
                case QMessageBox::Ok:
                    for(int j = 0; j < listeASCII.size(); j++)
                    {
                        progress.setValue(j);
                        if (progress.wasCanceled())
                        break;

                        i = listeASCII.at(j);

                        if(SPECIAL_CHAR.mid(0,4).contains(m_texte->at(i)))
                        {
                            m_texte->replace(i, 1, "e");
                        }
                        else if(SPECIAL_CHAR.mid(4,4).contains(m_texte->at(i)))
                        {
                            m_texte->replace(i, 1, "E");
                        }
                        else if(SPECIAL_CHAR.mid(8,4).contains(m_texte->at(i)))
                        {
                            m_texte->replace(i, 1, "a");
                        }
                        else if(SPECIAL_CHAR.mid(12,4).contains(m_texte->at(i)))
                        {
                            m_texte->replace(i, 1, "A");
                        }
                        else if(SPECIAL_CHAR.mid(16,4).contains(m_texte->at(i)))
                        {
                            m_texte->replace(i, 1, "o");
                        }
                        else if(SPECIAL_CHAR.mid(20,4).contains(m_texte->at(i)))
                        {
                            m_texte->replace(i, 1, "O");
                        }
                        else if(SPECIAL_CHAR.mid(24,4).contains(m_texte->at(i)))
                        {
                            m_texte->replace(i, 1, "i");
                        }
                        else if(SPECIAL_CHAR.mid(28,4).contains(m_texte->at(i)))
                        {
                            m_texte->replace(i, 1, "I");
                        }
                        else if(SPECIAL_CHAR.mid(32,4).contains(m_texte->at(i)))
                        {
                            m_texte->replace(i, 1, "u");
                        }
                        else if(SPECIAL_CHAR.mid(36,4).contains(m_texte->at(i)))
                        {
                            m_texte->replace(i, 1, "U");
                        }
                        else if(SPECIAL_CHAR.mid(41,1).contains(m_texte->at(i)))
                        {
                            m_texte->replace(i, 1, "C");
                        }
                        else if(SPECIAL_CHAR.mid(40,1).contains(m_texte->at(i)))
                        {
                            m_texte->replace(i, 1, "c");
                        }
                        else if(SPECIAL_CHAR.mid(42,1).contains(m_texte->at(i)))
                        {
                            m_texte->replace(i, 1, "n");
                        }
                        else if(SPECIAL_CHAR.mid(43,1).contains(m_texte->at(i)))
                        {
                            m_texte->replace(i, 1, "N");
                        }
                    }
                    progress.setValue(listeASCII.size());
                    return true;
                    break;

                case QMessageBox::Abort:
                    return false;
                    break;

                default:
                    sure = true;
                    break;
            }
        }
    }
    return true;
}

/// METHODES DE CHIFFREMENT

QString* Crypteur::chiffrer()
{
    QStringList methodes;
    methodes << tr("Chiffre de César") << tr("ROT13") << tr("ROTX") << tr("Chiffre de Vigenère") << tr("Chiffre de Vernam");

    bool ok;
    int choix = 0;
    QString methode = QInputDialog::getItem(m_parent, tr("Méthode"), tr("Choisissez une méthode de chiffrement : "), methodes, 0, false, &ok);

    if(ok && !methode.isEmpty())
    {
        if(methode == tr("Chiffre de César"))
        choix = 1;
        if(methode == tr("ROT13"))
        choix = 2;
        if(methode == tr("Chiffre de Vigenère"))
        choix = 3;
        if(methode == tr("Chiffre de Vernam"))
        choix = 4;
        if(methode == tr("ROTX"))
        choix = 5;

        int i = 0;

        switch(choix)
        {
            case 1:
                if(!cesarC())
                return NULL;
                break;

            case 2:
                if(!rot13C())
                return NULL;
                break;

            case 3:
                if(!vigenereC())
                return NULL;
                break;

            case 4:
                if(!vernamC())
                return NULL;
                break;

            case 5:
                bool ok;
                i = QInputDialog::getInt(this, tr("Rotation"), tr("Nombre de lettre de rotation : "), 1, 1, 25, 1, &ok);
                if (ok)
                {
                    if(!rot13C(i, 0))
                    return NULL;
                }
                break;

            default:
                break;
        }

        return m_texte;
    }
    return NULL;
}

            // SOUS METHODES DE CHIFFREMENT

            bool Crypteur::cesarC()
            {
                if(convertionAZaz())
                {
                    QProgressDialog progress(tr("Chiffrement en cours..."), tr("Annuler"), 0, m_texte->size(), m_parent);
                    progress.setWindowModality(Qt::WindowModal);
                    for(int i = 0; i < m_texte->size(); i++)
                    {
                        progress.setValue(i);
                        if (progress.wasCanceled())
                        {
                            return false;
                            break;
                        }

                        if(m_texte->at(i) == QChar('z'))
                        {
                            m_texte->replace(i, 1, QChar('a'));
                        }
                        else if(m_texte->at(i) == QChar('Z'))
                        {
                            m_texte->replace(i, 1, QChar('A'));
                        }
                        else if(((m_texte->at(i) >= QChar(65)) && (m_texte->at(i) <= QChar(90))) || ((m_texte->at(i) >= QChar(97)) && (m_texte->at(i) <= QChar(122))))
                        {
                            m_texte->replace(i, 1, QChar(m_texte->at(i).toLatin1() + 1));
                        }
                    }
                    progress.setValue(m_texte->size());
                    return true;
                }
                return false;
            }

            bool Crypteur::rot13C(int nombre, int methode)
            {
                if(convertionAZaz())
                {
                    QProgressDialog progress(m_parent);
                    switch(methode)
                    {
                        case 0:
                            progress.setLabelText(tr("Chiffrement en cours..."));
                            progress.setCancelButtonText(tr("Annuler"));
                            progress.setMinimum(0);
                            progress.setMaximum(m_texte->size());
                            break;

                        case 1:
                            progress.setLabelText(tr("Déchiffrement en cours..."));
                            progress.setCancelButtonText(tr("Annuler"));
                            progress.setMinimum(0);
                            progress.setMaximum(m_texte->size());
                            break;

                        case 2:
                            progress.setLabelText(tr("Décryptage en cours..."));
                            progress.setCancelButtonText(tr("Annuler"));
                            progress.setMinimum(0);
                            progress.setMaximum(m_texte->size());
                            break;

                        default:
                            progress.setLabelText(tr("Chiffrement en cours..."));
                            progress.setCancelButtonText(tr("Annuler"));
                            progress.setMinimum(0);
                            progress.setMaximum(m_texte->size());
                            break;
                    }
                    progress.setWindowModality(Qt::WindowModal);
                    for(int i = 0; i < m_texte->size(); i++)
                    {
                        progress.setValue(i);
                        if (progress.wasCanceled())
                        {
                            return false;
                            break;
                        }

                        if((m_texte->at(i) >= QChar(65)) && (m_texte->at(i) <= QChar(90)))
                        {
                            int caractere = m_texte->at(i).toLatin1();
                            caractere += nombre;

                            if(caractere > 90)
                            caractere -= 26;
                            else if(caractere < 65)
                            caractere += 26;

                            m_texte->replace(i, 1, QChar(caractere));
                        }
                        else if((m_texte->at(i) >= QChar(97)) && (m_texte->at(i) <= QChar(122)))
                        {
                            int caractere = m_texte->at(i).toLatin1();
                            caractere += nombre;

                            if(caractere > 122)
                            caractere -= 26;
                            else if(caractere < 97)
                            caractere += 26;

                            m_texte->replace(i, 1, QChar(caractere));
                        }
                    }
                    progress.setValue(m_texte->size());
                    return true;
                }
                return false;
            }

            bool Crypteur::vigenereC()
            {
                bool ok = true;
                QString cle;
                do{
                cle = QInputDialog::getText(this, tr("Clé"), tr("Choisissez la clé avec laquel le document sera chiffrer. \n Elle ne doit comporter que des lettres entre A et Z.\n La case n'a pas d'importance."), QLineEdit::Normal, "", &ok);
                }while(!AZazOnly(&cle));

                if (ok && !cle.isEmpty())
                {
                    if(convertionAZaz())
                    {
                        QProgressDialog progress(tr("Chiffrement en cours..."), tr("Annuler"), 0, m_texte->size(), m_parent);
                        progress.setWindowModality(Qt::WindowModal);
                        int j = 0;
                        for(int i = 0; i < m_texte->size(); i++)
                        {
                            progress.setValue(i);
                            if (progress.wasCanceled())
                            {
                                return false;
                                break;
                            }

                            if(m_texte->at(i) >= 65 && m_texte->at(i) <= 90)
                            {
                                int caractere = m_texte->at(i).toLatin1();
                                int caractereCle = cle.at(j).toUpper().toLatin1();
                                caractere += caractereCle;
                                caractere -= 65;

                                if(caractere > 90)
                                caractere -= 26;
                                if(caractere < 65)
                                caractere += 26;
                                j++;
                                if(j > (cle.count() - 1))
                                j = 0;

                                m_texte->replace(i, 1, QChar(caractere));
                            }
                            else if(m_texte->at(i) >= 97 && m_texte->at(i) <= 122)
                            {
                                int caractere = m_texte->at(i).toLatin1();
                                int caractereCle = cle.at(j).toLower().toLatin1();
                                caractere += caractereCle;
                                caractere -= 97;

                                if(caractere > 122)
                                caractere -= 26;
                                if(caractere < 97)
                                caractere += 26;
                                j++;
                                if(j > (cle.count() - 1))
                                j = 0;

                                m_texte->replace(i, 1, QChar(caractere));
                            }
                        }
                        progress.setValue(m_texte->size());
                        return true;
                    }
                }
                return false;
            }

            bool Crypteur::vernamC()
            {
                bool ok = true;
                bool perfect = convertionAZaz();
                QString cle;
                QProgressDialog progressCle(tr("Génération de la clé en cours..."), tr("Annuler"), 0, m_texte->size(), m_parent);
                progressCle.setWindowModality(Qt::WindowModal);
                for(int i = 0; i < compteLettreAZaz(); i++)
                {
                    progressCle.setValue(i);
                    if (progressCle.wasCanceled())
                    {
                        return false;
                        break;
                    }
                    cle.append(QChar(getRandomInt(65, 90)));
                }
                progressCle.setValue(m_texte->size());

                if (ok && !cle.isEmpty())
                {
                    if(perfect)
                    {
                        QProgressDialog progress(tr("Chiffrement en cours..."), tr("Annuler"), 0, m_texte->size(), m_parent);
                        progress.setWindowModality(Qt::WindowModal);
                        int j = 0;
                        for(int i = 0; i < m_texte->size(); i++)
                        {
                            progress.setValue(i);
                            if(progress.wasCanceled())
                            {
                                return false;
                                break;
                            }

                            if(m_texte->at(i) >= 65 && m_texte->at(i) <= 90)
                            {
                                int caractere = m_texte->at(i).toLatin1();
                                int caractereCle = cle.at(j).toUpper().toLatin1();
                                caractere += caractereCle;
                                caractere -= 65;

                                if(caractere > 90)
                                caractere -= 26;
                                if(caractere < 65)
                                caractere += 26;

                                m_texte->replace(i, 1, QChar(caractere));
                                j++;
                            }
                            else if(m_texte->at(i) >= 97 && m_texte->at(i) <= 122)
                            {
                                int caractere = m_texte->at(i).toLatin1();
                                int caractereCle = cle.at(j).toLower().toLatin1();
                                caractere += caractereCle;
                                caractere -= 97;

                                if(caractere > 122)
                                caractere -= 26;
                                if(caractere < 97)
                                caractere += 26;

                                m_texte->replace(i, 1, QChar(caractere));
                                j++;
                            }
                        }
                        progress.setValue(m_texte->size());
                        QMessageBox message(m_parent);
                        message.setIcon(QMessageBox::Question);
                        message.setWindowTitle(tr("Sauvegarde de la clé"));
                        message.setStandardButtons(QMessageBox::Save);
                        message.setText("Vous devez enregistrer la clé produite");
                        message.exec();
                        int onglet = m_parent->ongletActuel();
                        m_parent->creerDocument();
                        m_parent->documentActuel()->setPlainText(cle);
                        m_parent->enregistrerDocumentSous();
                        m_parent->setOnglets(onglet);
                        return true;
                    }
                }
                return false;
            }

/// METHODE DE DECHIFFREMENT

QString* Crypteur::dechiffrer()
{
    QStringList methodes;
    methodes << tr("Chiffre de César") << tr("ROT13") << tr("ROTX") << tr("Chiffre de Vigenère") << tr("Chiffre de Vernam");

    bool ok;
    int choix = 0;
    QString methode = QInputDialog::getItem(m_parent, tr("Méthode"), tr("Le fichier a été chiffrer à l'aide de quelle méthode?"), methodes, 0, false, &ok);

    if(ok && !methode.isEmpty())
    {
        if(methode == tr("Chiffre de César"))
        choix = 1;
        if(methode == tr("ROT13"))
        choix = 2;
        if(methode == tr("Chiffre de Vigenère"))
        choix = 3;
        if(methode == tr("Chiffre de Vernam"))
        choix = 4;
        if(methode == tr("ROTX"))
        choix = 5;

        int i = 0;

        switch(choix)
        {
            case 1:
                if(!cesarD())
                return NULL;
                break;

            case 2:
                if(!rot13D())
                return NULL;
                break;

            case 3:
                if(!vigenereD())
                return NULL;
                break;

            case 4:
                if(!vernamD())
                return NULL;
                break;

            case 5:
                bool ok;
                i = QInputDialog::getInt(this, tr("Rotation"), tr("Nombre de lettre de rotation : "), 1, 1, 25, 1, &ok);
                if (ok)
                {
                    if(!rot13C(-i, 1))
                    return NULL;
                }
                break;

            default:
                break;
        }
        return m_texte;
    }
    return NULL;
}

            // SOUS METHODES DE DECHIFFREMENT

            bool Crypteur::cesarD()
            {
                if(convertionAZaz())
                {
                    QProgressDialog progress(tr("Déchiffrement en cours..."), tr("Annuler"), 0, m_texte->size(), m_parent);
                    progress.setWindowModality(Qt::WindowModal);
                    for(int i = 0; i < m_texte->size(); i++)
                    {
                        progress.setValue(i);
                        if (progress.wasCanceled())
                        {
                            return false;
                            break;
                        }

                        if(m_texte->at(i) == QChar('a'))
                        {
                            m_texte->replace(i, 1, QChar('z'));
                        }
                        else if(m_texte->at(i) == QChar('A'))
                        {
                            m_texte->replace(i, 1, QChar('Z'));
                        }
                        else if(((m_texte->at(i) >= QChar(65)) && (m_texte->at(i) <= QChar(90))) || ((m_texte->at(i) >= QChar(97)) && (m_texte->at(i) <= QChar(122))))
                        {
                            m_texte->replace(i, 1, QChar(m_texte->at(i).toLatin1() - 1));
                        }
                    }
                    progress.setValue(m_texte->size());
                    return true;
                }
                return false;
            }

            bool Crypteur::rot13D()
            {
                return rot13C(13, 1);
            }

            bool Crypteur::vigenereD()
            {
                bool ok = true;
                QString cle;
                do{
                cle = QInputDialog::getText(this, tr("Clé"), tr("Choisissez la clé avec laquel le document a été chiffrer. \n Elle ne doit comporter que des lettres entre A et Z.\n La case n'a pas d'importance."), QLineEdit::Normal, "", &ok);
                }while(!AZazOnly(&cle));

                if (ok && !cle.isEmpty())
                {
                    if(convertionAZaz())
                    {
                        QProgressDialog progress(tr("Déchiffrement en cours..."), tr("Annuler"), 0, m_texte->size(), m_parent);
                        progress.setWindowModality(Qt::WindowModal);
                        int j = 0;
                        for(int i = 0; i < m_texte->size(); i++)
                        {
                            progress.setValue(i);
                            if (progress.wasCanceled())
                            {
                                return false;
                                break;
                            }

                            if(m_texte->at(i) >= 65 && m_texte->at(i) <= 90)
                            {
                                int caractere = m_texte->at(i).toLatin1();
                                int caractereCle = cle.at(j).toUpper().toLatin1();
                                caractere -= caractereCle;
                                caractere += 65;

                                if(caractere > 90)
                                caractere -= 26;
                                if(caractere < 65)
                                caractere += 26;
                                j++;
                                if(j > (cle.count() - 1))
                                j = 0;

                                m_texte->replace(i, 1, QChar(caractere));
                            }
                            else if(m_texte->at(i) >= 97 && m_texte->at(i) <= 122)
                            {
                                int caractere = m_texte->at(i).toLatin1();
                                int caractereCle = cle.at(j).toLower().toLatin1();
                                caractere -= caractereCle;
                                caractere += 97;

                                if(caractere > 122)
                                caractere -= 26;
                                if(caractere < 97)
                                caractere += 26;
                                j++;
                                if(j > (cle.count() - 1))
                                j = 0;

                                m_texte->replace(i, 1, QChar(caractere));
                            }
                        }
                        progress.setValue(m_texte->size());
                        return true;
                    }
                }
                return false;
            }

            bool Crypteur::vernamD()
            {
                bool perfect = convertionAZaz();
                QMessageBox message(m_parent);
                message.setWindowTitle(tr("Choix d'une clé"));
                message.setStandardButtons(QMessageBox::Ok);
                message.setText(tr("Vous devez choisir l'emplacement de la clé."));
                message.exec();
                bool pareil = true;
                QString cle;
                QString emplacement;
                do{
                    cle = "";
                    emplacement = "";
                    if(!pareil)
                    {
                        QMessageBox warning(m_parent);

                        warning.setWindowTitle(tr("Non-Compatible"));
                        warning.setIcon(QMessageBox::Warning);
                        warning.setStandardButtons(QMessageBox::Ok);
                        warning.setText(tr("La clé choisie ne contient pas le meme nombre de lettre non-ASCII que le document chiffré."));
                        warning.exec();
                    }

                    emplacement = QFileDialog::getOpenFileName(this, tr("Choix de la clé"), tr("sauvegarde\\"), tr("Fichiers textes (*.txt)"));

                    if(!emplacement.isEmpty())
                    {
                        QFile *fichier = new QFile(emplacement);
                        if (!fichier->open(QIODevice::ReadOnly))
                        {

                        }
                        else
                        {
                            QTextStream stream(fichier);
                            cle = stream.readAll();
                        }
                    }

                    pareil = false;
                }while((cle.count() != compteLettreAZaz()) && (!emplacement.isEmpty()));


                if (!cle.isEmpty())
                {
                    if(perfect)
                    {
                        QProgressDialog progress(tr("Déchiffrement en cours..."), tr("Annuler"), 0, m_texte->size(), m_parent);
                        progress.setWindowModality(Qt::WindowModal);
                        int j = 0;
                        for(int i = 0; i < m_texte->size(); i++)
                        {
                            progress.setValue(i);
                            if(progress.wasCanceled())
                            {
                                return false;
                                break;
                            }

                            if(m_texte->at(i) >= 65 && m_texte->at(i) <= 90)
                            {
                                int caractere = m_texte->at(i).toLatin1();
                                int caractereCle = cle.at(j).toUpper().toLatin1();
                                caractere -= caractereCle;
                                caractere += 65;

                                if(caractere > 90)
                                caractere -= 26;
                                if(caractere < 65)
                                caractere += 26;

                                m_texte->replace(i, 1, QChar(caractere));
                                j++;
                            }
                            else if(m_texte->at(i) >= 97 && m_texte->at(i) <= 122)
                            {
                                int caractere = m_texte->at(i).toLatin1();
                                int caractereCle = cle.at(j).toLower().toLatin1();
                                caractere -= caractereCle;
                                caractere += 97;

                                if(caractere > 122)
                                caractere -= 26;
                                if(caractere < 97)
                                caractere += 26;

                                m_texte->replace(i, 1, QChar(caractere));
                                j++;
                            }
                        }
                        progress.setValue(m_texte->size());
                        return true;
                    }
                }
                return false;
            }

/// METHODE DE DECRYPTION

void Crypteur::decrypter()
{
    if(convertionAZaz())
    {
        decrypteur = new Decrypteur(m_texte, m_parent);
        decrypteur->setWindowModality(Qt::WindowModal);
        decrypteur->setWindowFlags(Qt::Dialog);
        decrypteur->show();
    }
}
