/*
<This program is a text editor ebable of transcription on text file.>
<Ce programme est un éditeur de texte capable de chiffrement sur des fichier texte.>
Copyright (C) 2008 Keven Chaussé
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <QProgressDialog>

#include "Decrypteur.h"

Decrypteur::Decrypteur(QString *texte, FenCryptographe *parent) : QWidget(parent)
{
    Ui_Decrypteur::setupUi(this);
    m_texteOriginal = *texte;
    m_texteWait = new QString(*texte);
    m_texte = texte;
    m_parent = parent;

    modelFrequence = new QStandardItemModel;

    QStringList header;
    for(int i = 0; i <= 25; i++)
    {
        header << QChar(' ');
    }

    for(int i = 0; i <= 25; i++)
    {
        modelFrequence->setItem(0, i, new QStandardItem(QVariant(QChar(65 + i)).toString()));
    }
    modelFrequence->setHorizontalHeaderLabels(header);
    header.clear();
    header << tr("Ordre Alphabétique") << tr("Nombre d'apparition") << tr("Pourcentage d'apparition") << tr("Ordre Décroissant") << tr("Nombre d'apparition") << tr("Pourcentage d'apparition");
    modelFrequence->setVerticalHeaderLabels(header);
    header.clear();

    modelKasiski = new QStandardItemModel;

    frequenceTable->setModel(modelFrequence);
    kasiskiTable->setModel(modelKasiski);

    kasiskiTable->resizeColumnsToContents();
    kasiskiTable->resizeRowsToContents();
    frequenceTable->resizeColumnsToContents();
    frequenceTable->resizeRowsToContents();

    connect(analyser, SIGNAL(clicked()), this, SLOT(analyserdocument()));
    connect(calculer, SIGNAL(clicked()), this, SLOT(calcul()));
    connect(decrypterMono, SIGNAL(clicked()), this, SLOT(rotX()));
    connect(trouverCle, SIGNAL(clicked()), this, SLOT(calculerCle()));
}

void Decrypteur::calculerCle()
{
    QString stringTemp;
    QString cle;

    for(int i = 0; i < cleRot->value(); i++)
    {
        for(int j = 0; j <  m_texte->count() / cleRot->value(); j++)
        {
            stringTemp.append(m_texte->at(j * cleRot->value()));
        }

        QList<int> listeAlphabet;
        for(int i = 0; i < 26; i++)
        {
            listeAlphabet.append(alphabet[i]);
        }
        int nombreTotal = compteLettreAZaz();
        qSort(listeAlphabet.begin(), listeAlphabet.end());
        QChar lettre;
        double pourcentage = 0;

        for(int i = 0; i < 26; i++)
        {
            if(alphabet[i] != 0)
            {
                pourcentage = (double) 100 * ((double)alphabet[i] / (double)nombreTotal);
            }
            else
            {
                pourcentage = 0;
            }

            if(listeAlphabet.at(25) == alphabet[i])
            {
                lettre = QChar(i + 65);
            }
        }
        lettre = QChar(lettre.toLatin1() - 4 );

        if(lettre.toLatin1() < 65)
        lettre = QChar(lettre.toLatin1() + 26);

        cle.append(lettre);

        stringTemp.clear();
    }
    longueurCle->setText(cle);
}

void Decrypteur::rotX()
{
    m_texte = m_texteWait;
    QProgressDialog progress(m_parent);
    progress.setLabelText(tr("Décryptage en cours..."));
    progress.setCancelButtonText(tr("Annuler"));
    progress.setMinimum(0);
    progress.setMaximum(m_texte->size());
    progress.setWindowModality(Qt::WindowModal);
    for(int i = 0; i < m_texte->size(); i++)
    {
        progress.setValue(i);
        if (progress.wasCanceled())
        {
            break;
        }

        if((m_texte->at(i) >= QChar(65)) && (m_texte->at(i) <= QChar(90)))
        {
            int caractere = m_texte->at(i).toLatin1();
            caractere -= rotation->value();

            if(caractere > 90)
            caractere -= 26;
            else if(caractere < 65)
            caractere += 26;

            m_texte->replace(i, 1, QChar(caractere));
        }
        else if((m_texte->at(i) >= QChar(97)) && (m_texte->at(i) <= QChar(122)))
        {
            int caractere = m_texte->at(i).toLatin1();
            caractere -= rotation->value();

            if(caractere > 122)
            caractere -= 26;
            else if(caractere < 97)
            caractere += 26;

            m_texte->replace(i, 1, QChar(caractere));
        }
    }
    progress.setValue(m_texte->size());
    m_parent->creerDocument();
    m_parent->documentActuel()->setPlainText(*m_texte);
    this->close();
}

void Decrypteur::calcul()
{
    double IC = indiceCoincidence();
    QString ICString;
    ICString.setNum(IC, 'f', 4);
    ICLine->setText(ICString);
    analyser->setEnabled(true);
}

void Decrypteur::analyserdocument()
{
    if(analyseFrequentielle->isChecked())
    {
        analyseFrequence();
        chiffrementMono->setEnabled(true);
    }
    if(testDeKasiski->isChecked())
    {
        analyseKasiski();
        chiffrementPoly->setEnabled(true);
    }
}

int Decrypteur::compteLettreAZaz()
{
    int grandeur = 0;
    for(int i = 0; i < m_texte->size(); i++)
    {
        int caractere = m_texte->at(i).toLatin1();
        if(((caractere >= 65) && (caractere <= 90)) || ((caractere >= 97) && (caractere <= 122)))
        grandeur++;
    }
    return grandeur;
}

void Decrypteur::analyseKasiski()
{
    QProgressDialog progress(tr("Test de Kasiski en cours..."), tr("Annuler"), 0, m_texte->size() - 2, m_parent);
    progress.setWindowModality(Qt::WindowModal);
    QString buffer;
    QString buffer2;
    QStringList repetitionList;
    QList<int> distanceList;
    int distance = 0;
    int a = 0;

    m_texte->clear();
    m_texteOriginal = m_texteOriginal.toUpper();
    for(int i = 0; i < m_texteOriginal.size();  i++)
    {
        if(m_texteOriginal.at(i).toLatin1() >= 65 && m_texteOriginal.at(i).toLatin1() <= 90)
        {
            m_texte->append(m_texteOriginal.at(i));
        }
    }


    for(int i = 0; i < m_texte->size();  i++)
    {
        progress.setValue(i);
        if (progress.wasCanceled())
        {
            break;
        }

        if(i + 2 < m_texte->size())
        {
            buffer += QVariant(m_texte->at(i)).toString() += QVariant(m_texte->at(i + 1)).toString() += QVariant(m_texte->at(i + 2)).toString();

            for(int j = i + 1; j < m_texte->size(); j++)
            {
                distance++;
                if(j + 2 < m_texte->size())
                {
                    buffer2 += QVariant(m_texte->at(j)).toString() += QVariant(m_texte->at(j + 1)).toString() += QVariant(m_texte->at(j + 2)).toString();

                    if(buffer == buffer2)
                    {
                        for(a = 0; a < 7; a++)
                        {
                            if(i + 3 + a < m_texte->size() && j + 3 + a < m_texte->size())
                            {
                                if(m_texte->at(i + 3 + a) == m_texte->at(j + 3 + a))
                                {
                                    buffer += QVariant(m_texte->at(i + 3 + a)).toString();
                                }
                            }
                            else
                            {
                                break;
                            }
                        }
                        i += a;
                        repetitionList.append(buffer);
                        distanceList.append(distance);
                        break;
                    }
                    buffer2.clear();
                }
                else
                {
                    break;
                }
            }

            distance = 0;
            buffer.clear();
        }
        else
        {
            break;
        }
    }
    progress.setValue(m_texte->size() - 2);

    int nombre = 0;
    QList<int> longueurListe;
    QList<int> possibiliteListe;

    for(int i = 0; i < repetitionList.count(); i++)
    {
        modelKasiski->setItem(i, 0, new QStandardItem(repetitionList.at(i)));
        modelKasiski->setItem(i, 1, new QStandardItem(QVariant(distanceList.at(i)).toString()));

        nombre = QVariant(distanceList.at(i)).toInt();
        for(int j = 2; j <= (QVariant(distanceList.at(i)).toInt() / 2); j++)
        {
            if((nombre % j) == 0)
            {
                longueurListe.append(j);
            }
        }

        for(int j = 0; j < longueurListe.count(); j++)
        {
            if(qFind(possibiliteListe.begin(), possibiliteListe.end(), longueurListe.at(j)) == possibiliteListe.end())
            possibiliteListe += longueurListe.at(j);
        }
        longueurListe.clear();
    }
    qSort(possibiliteListe.begin(), possibiliteListe.end());

    QStringList header;
    header << tr("Séquence répétée") << tr("Distance entre\nles répétitions");

    for(int w = 0; w < possibiliteListe.count(); w++)
    {
        header.append(QVariant(possibiliteListe.at(w)).toString() += " lettres");
    }
    modelKasiski->setHorizontalHeaderLabels(header);

    for(int i = 0; i < repetitionList.count(); i++)
    {
        nombre = QVariant(distanceList.at(i)).toInt();
        for(int j = 2; j <= (QVariant(distanceList.at(i)).toInt() / 2); j++)
        {
            if((nombre % j) == 0)
            longueurListe.append(j);
        }

        for(int j = 0; j < longueurListe.count(); j++)
        {
            for(int w = 0; w < possibiliteListe.count(); w++)
            {
                if(possibiliteListe.at(w) == longueurListe.at(j))
                {
                    modelKasiski->setItem(i, w + 2, new QStandardItem("X"));
                }
            }
        }
        longueurListe.clear();
    }

    int nombreX = 0;
    int nombreXbuffer = 0;
    int longueurCle = 0;

    QModelIndex modelIndex;
    QVariant dataX;
    for(int i = 0; i < possibiliteListe.count(); i++)
    {
        for(int j = 0; j < repetitionList.count(); j++)
        {
            modelIndex = modelKasiski->index(j, i + 2);
            dataX = modelKasiski->data(modelIndex);
            if(dataX.toString() == "X")
            nombreXbuffer++;
        }
        if(nombreX < nombreXbuffer)
        {
            nombreX = nombreXbuffer;
            longueurCle = possibiliteListe.at(i);
        }
        nombreXbuffer = 0;
    }

    LineCle->setText(QVariant(longueurCle).toString());

    kasiskiTable->resizeColumnsToContents();
    kasiskiTable->resizeRowsToContents();
}

double Decrypteur::indiceCoincidence()
{
    QProgressDialog progress(tr("Calcul de l'indice de coïncidence en cours..."), tr("Annuler"), 0, m_texte->size(), m_parent);
    progress.setWindowModality(Qt::WindowModal);

    for(int i = 0; i < 26; i++)
    {
        alphabet[i] = 0;
    }

    for(int i = 0; i < m_texte->size(); i++)
    {
        progress.setValue(i);
        if (progress.wasCanceled())
        {
            return 0;
            break;
        }

        if(m_texte->at(i) == QChar(65) || m_texte->at(i) == QChar(97))
        alphabet[0]++;
        else if(m_texte->at(i) == QChar(66) || m_texte->at(i) == QChar(98))
        alphabet[1]++;
        else if(m_texte->at(i) == QChar(67) || m_texte->at(i) == QChar(99))
        alphabet[2]++;
        else if(m_texte->at(i) == QChar(68) || m_texte->at(i) == QChar(100))
        alphabet[3]++;
        else if(m_texte->at(i) == QChar(69) || m_texte->at(i) == QChar(101))
        alphabet[4]++;
        else if(m_texte->at(i) == QChar(70) || m_texte->at(i) == QChar(102))
        alphabet[5]++;
        else if(m_texte->at(i) == QChar(71) || m_texte->at(i) == QChar(103))
        alphabet[6]++;
        else if(m_texte->at(i) == QChar(72) || m_texte->at(i) == QChar(104))
        alphabet[7]++;
        else if(m_texte->at(i) == QChar(73) || m_texte->at(i) == QChar(105))
        alphabet[8]++;
        else if(m_texte->at(i) == QChar(74) || m_texte->at(i) == QChar(106))
        alphabet[9]++;
        else if(m_texte->at(i) == QChar(75) || m_texte->at(i) == QChar(107))
        alphabet[10]++;
        else if(m_texte->at(i) == QChar(76) || m_texte->at(i) == QChar(108))
        alphabet[11]++;
        else if(m_texte->at(i) == QChar(77) || m_texte->at(i) == QChar(109))
        alphabet[12]++;
        else if(m_texte->at(i) == QChar(78) || m_texte->at(i) == QChar(110))
        alphabet[13]++;
        else if(m_texte->at(i) == QChar(79) || m_texte->at(i) == QChar(111))
        alphabet[14]++;
        else if(m_texte->at(i) == QChar(80) || m_texte->at(i) == QChar(112))
        alphabet[15]++;
        else if(m_texte->at(i) == QChar(81) || m_texte->at(i) == QChar(113))
        alphabet[16]++;
        else if(m_texte->at(i) == QChar(82) || m_texte->at(i) == QChar(114))
        alphabet[17]++;
        else if(m_texte->at(i) == QChar(83) || m_texte->at(i) == QChar(115))
        alphabet[18]++;
        else if(m_texte->at(i) == QChar(84) || m_texte->at(i) == QChar(116))
        alphabet[19]++;
        else if(m_texte->at(i) == QChar(85) || m_texte->at(i) == QChar(117))
        alphabet[20]++;
        else if(m_texte->at(i) == QChar(86) || m_texte->at(i) == QChar(118))
        alphabet[21]++;
        else if(m_texte->at(i) == QChar(87) || m_texte->at(i) == QChar(119))
        alphabet[22]++;
        else if(m_texte->at(i) == QChar(88) || m_texte->at(i) == QChar(120))
        alphabet[23]++;
        else if(m_texte->at(i) == QChar(89) || m_texte->at(i) == QChar(121))
        alphabet[24]++;
        else if(m_texte->at(i) == QChar(90) || m_texte->at(i) == QChar(122))
        alphabet[25]++;
    }
    progress.setValue(m_texte->size());
    double nombreTotal = (double) compteLettreAZaz();
    double IC = 0;
    for(int i = 0; i < 26; i++)
    {
        IC += (((double) alphabet[i] * ((double) alphabet[i] - 1))/(nombreTotal * (nombreTotal - 1)));
    }
    return IC;
}

void Decrypteur::analyseFrequence()
{
    QList<int> listeAlphabet;
    for(int i = 0; i < 26; i++)
    {
        listeAlphabet.append(alphabet[i]);
    }
    int nombreTotal = compteLettreAZaz();
    qSort(listeAlphabet.begin(), listeAlphabet.end());
    QString nombre;
    QString lettre;
    double pourcentage = 0;

    for(int i = 0; i < 26; i++)
    {
        nombre.setNum(alphabet[i]);
        modelFrequence->setItem(1, i, new QStandardItem(nombre));
        if(alphabet[i] != 0)
        {
            pourcentage = (double) 100 * ((double)alphabet[i] / (double)nombreTotal);
            nombre.setNum(pourcentage, 'f', 2);
        }
        else
        {
            pourcentage = 0;
            nombre.setNum(pourcentage, 'f', 2);
        }
        nombre += "%";
        modelFrequence->setItem(2, i, new QStandardItem(nombre));

        if(listeAlphabet.at(25) == alphabet[i])
        {
            lettre = QChar(i + 65);
        }
    }
    int difference = lettre.at(0).toLatin1() - 69;

    if(difference < 0)
    difference *= -1;
    nombre.setNum(difference);
    frequenceLine->setText(nombre);
    int x = 0;

    for(int i = 25; i >= 0; i--)
    {
        for(int j = 0; j < 26; j++)
        {
            if(listeAlphabet.at(i) == alphabet[j])
            {
                modelFrequence->setItem(3, x, new QStandardItem(QVariant(QChar(j + 65)).toString()));
                nombre.setNum(alphabet[j]);
                modelFrequence->setItem(4, x, new QStandardItem(nombre));
                if(alphabet[j] != 0)
                {
                    pourcentage = (double) 100 * ((double)alphabet[j] / (double)nombreTotal);
                    nombre.setNum(pourcentage, 'f', 2);
                }
                else
                {
                    pourcentage = 0;
                    nombre.setNum(pourcentage, 'f', 2);
                }
                nombre += "%";
                modelFrequence->setItem(5, x, new QStandardItem(nombre));
                alphabet[j] = -1;
                x++;
                break;
            }
        }
    }

    frequenceTable->resizeColumnsToContents();
    frequenceTable->resizeRowsToContents();
}
