/*
<This program is a text editor ebable of transcription on text file.>
<Ce programme est un éditeur de texte capable de chiffrement sur des fichier texte.>
Copyright (C) 2008 Keven Chaussé
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <QFileDialog>
#include <QPrinter>
#include <QPrintDialog>
#include <QInputDialog>

#include "Crypteur.h"

/// CONSTRUCTEUR PRINCIPAL

FenCryptographe::FenCryptographe(QMainWindow *parent) : QMainWindow(parent)
{
    crypto = new QDialog;
    Ui::Cryptopedia cryp;
    cryp.setupUi(crypto);

    //Préparation de l'interface
    setupUi(this);
    zoneCentrale = new QWidget;
    onglets = new QTabWidget;
    listeEmplacement = new QList<QString>;
    modificationFichier = new QList<bool>;
    nomModifier = new QList<bool>;
    isRedoAvailable = new QList<bool>;
    isUndoAvailable = new QList<bool>;
    isCopyAvailable = new QList<bool>;
    nombreFichier = 0;

    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(onglets);
    zoneCentrale->setLayout(layout);
    onglets->setVisible(false);


    setCentralWidget(zoneCentrale);

    creerActions();
    creerBarreEtat();
    setEnabledAction(false);

    actionRepeter->setEnabled(false);
    actionAnnuler->setEnabled(false);
    actionCouper->setEnabled(false);
    actionCopier->setEnabled(false);
    actionSupprimer->setEnabled(false);
    actionEnregistrer->setEnabled(false);

    connect(this, SIGNAL(ongletsExiste(bool)), onglets, SLOT(setVisible(bool)));
    connect(onglets, SIGNAL(currentChanged(int)), this, SLOT(changementOnglets(int)));
}





/// FONCTIONS D'INITIALISATION

void FenCryptographe::creerActions()
{
    connect(actionNouveau, SIGNAL(triggered()), this, SLOT(creerDocument()));
    connect(actionOuvrir, SIGNAL(triggered()), this, SLOT(ouvrirDocument()));
    connect(actionEnregistrer, SIGNAL(triggered()), this, SLOT(enregistrerDocument()));
    connect(actionEnregistrerSous, SIGNAL(triggered()), this, SLOT(enregistrerDocumentSous()));
    connect(actionFermerFichier, SIGNAL(triggered()), this, SLOT(verificationFichierSave()));
    connect(actionFermerToutFichiers, SIGNAL(triggered()), this, SLOT(fermerToutDocuments()));
    connect(actionQuitter, SIGNAL(triggered()), this, SLOT(close()));
    connect(actionAnnuler, SIGNAL(triggered()), this, SLOT(annuler()));
    connect(actionRepeter, SIGNAL(triggered()), this, SLOT(repeter()));
    connect(actionCouper, SIGNAL(triggered()), this, SLOT(couper()));
    connect(actionCopier, SIGNAL(triggered()), this, SLOT(copier()));
    connect(actionColler, SIGNAL(triggered()), this, SLOT(coller()));
    connect(actionSelectionnerTout, SIGNAL(triggered()), this, SLOT(selectionnerTout()));
    connect(actionRechercher, SIGNAL(triggered()), this, SLOT(rechercher()));
    connect(actionPreferences, SIGNAL(triggered()), this, SLOT(changerPreferences()));
    connect(actionChiffrer, SIGNAL(triggered()), this, SLOT(chiffrer()));
    connect(actionDechiffrer, SIGNAL(triggered()), this, SLOT(dechiffrer()));
    connect(actionDecrypter, SIGNAL(triggered()), this, SLOT(decrypter()));
    connect(actionCryptopedia, SIGNAL(triggered()), this, SLOT(informationCrypto()));
    connect(actionImprimer, SIGNAL(triggered()), this, SLOT(imprimer()));
    connect(actionAProposDeQt, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
    connect(actionAProposDeKC, SIGNAL(triggered()), this, SLOT(aboutKC()));
    connect(actionOrdreAlpha, SIGNAL(triggered()), this, SLOT(ordreAlpha()));
}

void FenCryptographe::creerBarreEtat()
{
    progression = new QProgressBar(this);
    progression->setVisible(false);
    progression->setMaximumHeight(14);
    Ui_FenCryptographe::statusBar->addWidget(progression, 1);
}





/// AUTRES FONCTIONS

QTextEdit* FenCryptographe::documentActuel()
{
    return onglets->currentWidget()->findChild<QTextEdit *>();
}

int FenCryptographe::ongletActuel()
{
    return onglets->currentIndex();
}

void FenCryptographe::setOnglets(int onglet)
{
    onglets->setCurrentIndex(onglet);
}

void FenCryptographe::modifierFichier()
{
    if(!modificationFichier->at(onglets->currentIndex()) && !nomModifier->at(onglets->currentIndex()))
    {
        QString nomFichier = "*";
        nomFichier += onglets->tabText(onglets->currentIndex());
        onglets->setTabText(onglets->currentIndex(), nomFichier);
        nomModifier->replace(onglets->currentIndex(), true);
        modificationFichier->replace(onglets->currentIndex(), true);
        actionEnregistrer->setEnabled(true);
    }
}

void FenCryptographe::setEnabledAction(bool active)
{
    actionChiffrer->setEnabled(active);
    actionDechiffrer->setEnabled(active);
    actionDecrypter->setEnabled(active);
    actionFermerToutFichiers->setEnabled(active);
    actionFermerFichier->setEnabled(active);
    actionImprimer->setEnabled(active);
    actionRechercher->setEnabled(active);
    actionSelectionnerTout->setEnabled(active);
    actionEnregistrer->setEnabled(active);
    actionEnregistrerSous->setEnabled(active);
    actionColler->setEnabled(active);
    actionImprimer->setEnabled(active);
    actionOrdreAlpha->setEnabled(active);
}





/// SLOTS FICHIER

void FenCryptographe::creerDocument()
{
    modificationFichier->append(true);
    nomModifier->append(true);
    isUndoAvailable->append(false);
    isRedoAvailable->append(false);
    isCopyAvailable->append(false);

    QString emplacementDefaut(tr("sauvegarde\\"));
    QString nomFichier(tr("sanstitre"));
    QString complet;
    QFile fichier;
    do{
        complet.clear();
        nombreFichier++;

        complet += emplacementDefaut;
        complet += nomFichier;
        complet += QString::number(nombreFichier,10);
        complet += ".txt";

        fichier.setFileName(complet);
    }while(fichier.exists());

    complet.clear();
    complet += "*";
    nomFichier += QString::number(nombreFichier,10);
    nomFichier += ".txt";
    complet += nomFichier;

    onglets->addTab(creerOnglet(), complet);
    onglets->setCurrentIndex((int)onglets->count() - 1);

    listeEmplacement->append(tr("sauvegarde\\") += nomFichier);

    if(onglets->count() == 1)
    setEnabledAction(true);

    if(onglets->count() == 1)
    emit ongletsExiste(true);

    connect(documentActuel(), SIGNAL(textChanged()), this, SLOT(modifierFichier()));
    connect(documentActuel(), SIGNAL(redoAvailable(bool)), actionRepeter, SLOT(setEnabled(bool)));
    connect(documentActuel(), SIGNAL(undoAvailable(bool)), actionAnnuler, SLOT(setEnabled(bool)));
    connect(documentActuel(), SIGNAL(copyAvailable(bool)), actionCouper, SLOT(setEnabled(bool)));
    connect(documentActuel(), SIGNAL(copyAvailable(bool)), actionCopier, SLOT(setEnabled(bool)));
    connect(documentActuel(), SIGNAL(redoAvailable(bool)), this, SLOT(changementRedo(bool)));
    connect(documentActuel(), SIGNAL(undoAvailable(bool)), this, SLOT(changementUndo(bool)));
    connect(documentActuel(), SIGNAL(copyAvailable(bool)), this, SLOT(changementCopy(bool)));
}

void FenCryptographe::ouvrirDocument()
{
    QString nomFichier = QFileDialog::getOpenFileName(this, tr("Ouvrir un fichier texte"), tr("sauvegarde\\"), tr("Fichiers textes (*.txt)"));
    QFile *fichier = new QFile(nomFichier);

    if (!fichier->open(QIODevice::ReadWrite | QIODevice::Text))
    {

    }
    else
    {
        QFileInfo infoFichier(fichier->fileName());

        //Vérifie si le fichier est déja ouvert
        bool fichierExiste = false;
        for(int i = 0; i < onglets->count(); i++)
        {
            if(listeEmplacement->at(i) == infoFichier.filePath())
            fichierExiste = true;
        }

        if(!fichierExiste)
        {
            modificationFichier->append(false);
            nomModifier->append(false);
            isUndoAvailable->append(false);
            isRedoAvailable->append(false);
            isCopyAvailable->append(false);
            onglets->addTab(creerOnglet(fichier), infoFichier.fileName());
            onglets->setCurrentIndex(onglets->count() - 1);
            QTextStream stream(fichier);
            QString contenuFichier;
            contenuFichier = stream.readAll();

            if(onglets->count() == 1)
            emit ongletsExiste(true);
            documentActuel()->setPlainText(contenuFichier);

            connect(documentActuel(), SIGNAL(textChanged()), this, SLOT(modifierFichier()));
            connect(documentActuel(), SIGNAL(redoAvailable(bool)), actionRepeter, SLOT(setEnabled(bool)));
            connect(documentActuel(), SIGNAL(undoAvailable(bool)), actionAnnuler, SLOT(setEnabled(bool)));
            connect(documentActuel(), SIGNAL(copyAvailable(bool)), actionCouper, SLOT(setEnabled(bool)));
            connect(documentActuel(), SIGNAL(copyAvailable(bool)), actionColler, SLOT(setEnabled(bool)));
            connect(documentActuel(), SIGNAL(copyAvailable(bool)), actionCopier, SLOT(setEnabled(bool)));
            connect(documentActuel(), SIGNAL(redoAvailable(bool)), this, SLOT(changementRedo(bool)));
            connect(documentActuel(), SIGNAL(undoAvailable(bool)), this, SLOT(changementUndo(bool)));
            connect(documentActuel(), SIGNAL(copyAvailable(bool)), this, SLOT(changementCopy(bool)));
        }
        fichier->close();
        if(onglets->count() == 1)
        setEnabledAction(true);
    }
}

bool FenCryptographe::enregistrerDocument()
{
    QFile *fichier = new QFile(listeEmplacement->at(onglets->currentIndex()));
    if(fichier->exists())
    {
        if(!fichier->open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::Text))
        {
            return false;
        }
        else
        {
            QTextStream out(fichier);
            out << documentActuel()->toPlainText();
            fichier->close();
            QString nomFichier = onglets->tabText(onglets->currentIndex());
            nomFichier.remove(0,1);
            onglets->setTabText(onglets->currentIndex(), nomFichier);
            modificationFichier->replace(onglets->currentIndex(), false);
            nomModifier->replace(onglets->currentIndex(), false);
            actionEnregistrer->setEnabled(false);
            return true;
        }

    }
    else
    {
        QString nomFichier = QFileDialog::getSaveFileName(this, tr("Sauvegarder un fichier"), listeEmplacement->at(onglets->currentIndex()), tr("Fichiers textes (*.txt)"));
        fichier->setFileName(nomFichier);
        if(!fichier->open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::Text))
        {
            return false;
        }
        else
        {
            QTextStream out(fichier);
            out << documentActuel()->toPlainText();
            fichier->close();
            nomFichier = onglets->tabText(onglets->currentIndex());
            nomFichier.remove(0,1);
            onglets->setTabText(onglets->currentIndex(), nomFichier);
            modificationFichier->replace(onglets->currentIndex(), false);
            nomModifier->replace(onglets->currentIndex(), false);
            actionEnregistrer->setEnabled(false);
            listeEmplacement->replace(onglets->currentIndex(), fichier->fileName());
            QFileInfo infoFichier(fichier->fileName());
            onglets->setTabText(onglets->currentIndex(), infoFichier.fileName());
            return true;
        }
    }
}

void FenCryptographe::enregistrerDocumentSous()
{
    QString nomFichier = QFileDialog::getSaveFileName(this, tr("Sauvegarder un fichier"), listeEmplacement->at(onglets->currentIndex()), tr("Fichiers textes (*.txt)"));

    QFile *fichier = new QFile(nomFichier);
    if(!fichier->open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::Text))
    {

    }
    else
    {
        QTextStream out(fichier);
        out << documentActuel()->toPlainText();
        fichier->close();
        modificationFichier->replace(onglets->currentIndex(), false);
        nomModifier->replace(onglets->currentIndex(), false);
        actionEnregistrer->setEnabled(false);
        QFileInfo infoFichier(fichier->fileName());
        listeEmplacement->replace(onglets->currentIndex(), fichier->fileName());
        onglets->setTabText(onglets->currentIndex(), infoFichier.fileName());
    }
}

void FenCryptographe::fermerDocument(bool forcer)
{
    if(forcer)
    {
        for(int i = onglets->currentIndex(); i < onglets->count() - 1; i++)
        {
            listeEmplacement->swap(i, i + 1);
            modificationFichier->swap(i, i + 1);
            nomModifier->swap(i, i + 1);
            isRedoAvailable->swap(i, i + 1);
            isUndoAvailable->swap(i, i + 1);
            isCopyAvailable->swap(i, i + 1);
        }
        listeEmplacement->removeAt(onglets->count() - 1);
        modificationFichier->removeAt(onglets->count() - 1);
        nomModifier->removeAt(onglets->count() - 1);
        isRedoAvailable->removeAt(onglets->count() - 1);
        isUndoAvailable->removeAt(onglets->count() - 1);
        isCopyAvailable->removeAt(onglets->count() - 1);

        onglets->removeTab(onglets->currentIndex());

        if(onglets->count() == 0)
        setEnabledAction(false);

        if(onglets->count() == 0)
        emit ongletsExiste(false);
    }
}

void FenCryptographe::fermerToutDocuments()
{
    int nombreOnglets = onglets->count();

    for(int i = 0; i < nombreOnglets; i++)
    {
        onglets->setCurrentIndex(0);
        verificationFichierSave();
    }
}

void FenCryptographe::imprimer()
{
    QPrinter *printer = new QPrinter;
    QPrintDialog printerSetting(printer, this);
    QString nomFichier(onglets->tabText(onglets->currentIndex()));

    if(nomFichier.startsWith("*"))
    nomFichier.remove(0,1);

    nomFichier.remove(".txt");

    printer->setCreator("KCryptographe © Keven Chaussé");
    printer->setDocName(nomFichier);

    if(printerSetting.exec())
    documentActuel()->print(printer);
}


                // SOUS SLOTS FICHIER

                QWidget *FenCryptographe::creerOnglet(QFile* fichier)
                {
                    QWidget* pageOnglet = new QWidget;
                    QTextEdit* document = new QTextEdit;

                    if(fichier)
                    {
                        QString* emplacement = new QString(fichier->fileName());
                        listeEmplacement->append(emplacement->toLatin1());
                    }

                    QVBoxLayout *layout = new QVBoxLayout;
                    layout->setContentsMargins(0,0,0,0);
                    layout->addWidget(document);
                    pageOnglet->setLayout(layout);

                    return pageOnglet;
                }

                void FenCryptographe::changementOnglets(int current)
                {
                    if(onglets->count() > 0)
                    {
                        if(!modificationFichier->at(current))
                        {
                            actionEnregistrer->setEnabled(false);
                        }
                        else
                        {
                            actionEnregistrer->setEnabled(true);
                        }

                        if(!isUndoAvailable->at(current))
                        {
                            actionAnnuler->setEnabled(false);
                        }
                        else
                        {
                            actionAnnuler->setEnabled(true);
                        }

                        if(!isRedoAvailable->at(current))
                        {
                            actionRepeter->setEnabled(false);
                        }
                        else
                        {
                            actionRepeter->setEnabled(true);
                        }

                        if(!isCopyAvailable->at(current))
                        {
                            actionCopier->setEnabled(false);
                        }
                        else
                        {
                            actionCopier->setEnabled(true);
                        }

                        if(!isCopyAvailable->at(current))
                        {
                            actionCouper->setEnabled(false);
                        }
                        else
                        {
                            actionCouper->setEnabled(true);
                        }
                    }
                }

                bool FenCryptographe::verificationFichierSave()
                {
                    if(modificationFichier->at(onglets->currentIndex()))
                    {
                        QMessageBox message(this);
                        message.setIcon(QMessageBox::Question);
                        message.setWindowTitle(tr("Sauvegarder un fichier"));
                        message.setStandardButtons(QMessageBox::Yes | QMessageBox::No | QMessageBox::Abort);
                        QString messageString(tr("Le fichier "));
                        QString nomFichier(onglets->tabText(onglets->currentIndex()));
                        nomFichier.remove(0,1);
                        messageString += nomFichier;
                        messageString += tr(" n'a pas été sauvegardé! Voulez-vous l'enregistrer?");
                        message.setText(messageString);

                        switch(message.exec())
                        {
                            case QMessageBox::Yes:
                                if(enregistrerDocument())
                                {
                                    fermerDocument(true);
                                    return true;
                                }
                                else
                                {
                                    return false;
                                }
                                break;

                            case QMessageBox::No:
                                fermerDocument(true);
                                return true;
                                break;

                            case QMessageBox::Abort:
                                return false;
                                break;

                            default:
                                break;
                        }
                    }
                    else
                    {
                        fermerDocument(true);
                    }
                    return false;
                }





/// SLOTS ÉDITION

void FenCryptographe::annuler()
{
    documentActuel()->undo();
}

void FenCryptographe::repeter()
{
    documentActuel()->redo();
}

void FenCryptographe::couper()
{
    documentActuel()->cut();
}

void FenCryptographe::copier()
{
    documentActuel()->copy();
}

void FenCryptographe::coller()
{
    documentActuel()->paste();
}

void FenCryptographe::selectionnerTout()
{
    documentActuel()->selectAll();
}

void FenCryptographe::rechercher()
{
    bool ok;
    QString search = QInputDialog::getText(this, tr("Recherche"), tr("Rechercher le suivant :"), QLineEdit::Normal, "", &ok);

    if (ok && !search.isEmpty())
    {
        documentActuel()->find(search);
    }
}

void FenCryptographe::changerPreferences()
{

}


                // SOUS SLOTS ÉDITION

                void FenCryptographe::changementRedo(bool change)
                {
                    isRedoAvailable->replace(onglets->currentIndex(), change);
                }

                void FenCryptographe::changementUndo(bool change)
                {
                    isUndoAvailable->replace(onglets->currentIndex(), change);
                }

                void FenCryptographe::changementCopy(bool change)
                {
                    isCopyAvailable->replace(onglets->currentIndex(), change);
                }





/// SLOTS OUTILS

void FenCryptographe::chiffrer()
{
    if(documentActuel()->toPlainText() != "")
    {
        Crypteur crypteur(documentActuel()->toPlainText(), this);
        QString *wait = crypteur.chiffrer();

        if(wait)
        {
            documentActuel()->selectAll();
            documentActuel()->insertPlainText(*wait);
        }
    }
}

void FenCryptographe::dechiffrer()
{
    if(documentActuel()->toPlainText() != "")
    {
        Crypteur crypteur(documentActuel()->toPlainText(), this);
        QString *wait = crypteur.dechiffrer();

        if(wait)
        {
            documentActuel()->selectAll();
            documentActuel()->insertPlainText(*wait);
        }
    }
}

void FenCryptographe::decrypter()
{
    if(documentActuel()->toPlainText() != "")
    {
        Crypteur crypteur(documentActuel()->toPlainText(), this);
        crypteur.decrypter();
    }
}

void FenCryptographe::ordreAlpha()
{
    QString string = documentActuel()->toPlainText();
    QString buffer;
    QStringList listeString;

    if(!string.endsWith(QChar('\n')))
    string += QChar('\n');

    int i = 0;
    while(i < string.size())
    {
        while((string.at(i) != QChar('\n')) && (i < string.size()))
        {
            buffer += string.at(i);
            i++;
        }
        i++;
        buffer += QChar('\n');

        listeString.append(buffer);

        buffer.clear();
    }
    buffer.clear();
    listeString.sort();
    for(int i = 0; i < listeString.count(); i++)
    buffer += listeString.at(i);

    documentActuel()->setPlainText(buffer);
}





/// SLOTS INFORMATION

void FenCryptographe::informationCrypto()
{
    crypto->exec();
}

void FenCryptographe::aboutKC()
{
    QMessageBox message(this);
    message.setWindowTitle(tr("À propos de KCryptographe"));
    message.setStandardButtons(QMessageBox::Ok);
    message.setText(tr("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd\"><html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">p, li { white-space: pre-wrap; }</style></head><body style=\" font-family:'MS Shell Dlg 2'; font-size:8.25pt; font-weight:400; font-style:normal;\"><p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:12pt; font-weight:600; text-decoration: underline;\">À propos de KCryptographe</span></p><p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p><p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:9pt;\">&lt;This program is a text editor capable of encryption on text file.&gt;</span></p><p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:9pt;\">&lt;Ce programme est un éditeur de texte capable de chiffrement sur des fichier texte.&gt;</p><p align=\"center\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:9pt;\"></p><p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:9pt;\">Copyright (C) 2008 Keven Chaussé</p><p align=\"center\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:9pt;\"></p><p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:9pt;\">This program is free software: you can redistribute it and/or modify</p><p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:9pt;\">it under the terms of the GNU General Public License as published by</p><p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:9pt;\">the Free Software Foundation, either version 3 of the License, or</p><p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:9pt;\">(at your option) any later version.</p><p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:9pt;\">This program is distributed in the hope that it will be useful,</p><p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:9pt;\">but WITHOUT ANY WARRANTY; without even the implied warranty of</p><p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:9pt;\">MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the</p><p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:9pt;\">GNU General Public License for more details.</p><p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:9pt;\">You should have received a copy of the GNU General Public License</p><p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:9pt;\">along with this program. If not, see &lt;<a href=\"http://www.gnu.org/licenses/\"><span style=\" text-decoration: underline; color:#0000ff;\">GPL 3.0</span></a>&gt;.</p></body></html>"));
    message.exec();
}


/// MÉTHODES SURCHARGÉS

void FenCryptographe::closeEvent(QCloseEvent *event)
{
    bool quitter = true;
    if(onglets->currentIndex() != -1)
    {
        onglets->setCurrentIndex(0);
        int i = 0;
        int nombreOnglets = onglets->count() - 1;

        while(quitter && (i <= nombreOnglets))
        {
            if(modificationFichier->at(onglets->currentIndex()))
            {
                QMessageBox message(this);
                message.setIcon(QMessageBox::Question);
                message.setWindowTitle(tr("Sauvegarder un fichier"));
                message.setStandardButtons(QMessageBox::Yes | QMessageBox::No | QMessageBox::Abort);
                QString messageString(tr("Le fichier "));
                QString nomFichier(onglets->tabText(onglets->currentIndex()));
                nomFichier.remove(0,1);
                messageString += nomFichier;
                messageString += tr(" n'a pas été sauvegardé! Voulez-vous l'enregistrer?");
                message.setText(messageString);

                switch(message.exec())
                {
                    case QMessageBox::Yes:
                        if(enregistrerDocument())
                        {
                            fermerDocument(true);
                            quitter = true;
                        }
                        else
                        {
                            quitter = false;
                        }
                        break;

                    case QMessageBox::No:
                        fermerDocument(true);
                        quitter = true;
                        break;

                    case QMessageBox::Abort:
                        quitter = false;
                        break;

                    default:
                        break;
                }
            }
            else
            {
                fermerDocument(true);
            }
            i++;
        }
    }

    if(quitter == true)
    {
        event->accept();
    }
    else
    {
        event->ignore();
    }
}
